@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@ bool Basic_buffer::add_line( const char * const buf, const int len )
   if( data.back().size() > LONG_MAX / 2 ) return false;
   if( data.back().size() && data.back()[data.back().size()-1] == '\n' )
     {
-    if( data.size() >= data.capacity() && data.size() >= 10000 )
+    if( data.size() >= data.capacity() && data.size() >= 1000 )
       data.reserve( data.size() + ( data.size() / 10 ) );
     data.push_back( std::string() );
     }
@@ -75,8 +75,7 @@ bool Basic_buffer::detect_and_remove_cr()
   for( long i = 0; i < last_line(); ++i )
     {
     const std::string & s = data[i];
-    const long size = s.size();
-    if( size < 2 || s[size-2] != '\r' ) return false;
+    if( s.size() < 2 || s[s.size()-2] != '\r' ) return false;
     }
   for( long i = 0; i < last_line(); ++i )	// remove the CR before each LF
     data[i].erase( data[i].size() - 2, 1 );
@@ -95,15 +94,11 @@ Basic_buffer::Basic_buffer( const Basic_buffer & b,
 
 bool Basic_buffer::blank( const long line ) const
   {
-  Point p( line, 0 );
-  return ( !pisvalid( p ) || bot( p ) == eol( p ) );
-  }
-
-
-long Basic_buffer::characters( const long line ) const
-  {
-  if( line < 0 || line >= lines() ) return 0;
-  return data[line].size();
+  if( line < 0 || line >= lines() ) return true;
+  const std::string & s = data[line];
+  for( unsigned long i = 0; i < s.size(); ++i )
+    if( !ISO_8859::isspace( s[i] ) ) return false;
+  return true;
   }
 
 
@@ -117,33 +112,29 @@ bool Basic_buffer::one_character() const
 int Basic_buffer::operator[]( const Point & p ) const
   {
   if( p.col < 0 || p.col >= characters( p.line ) ) return -1;
-  unsigned char ch = data[p.line][p.col];
-  return ch;
+  return (unsigned char)data[p.line][p.col];
   }
 
 
-int Basic_buffer::pgetc( Point & p ) const
+int Basic_buffer::pgetc( Point & p ) const	// inlining makes it slower
   {
-  Point p1 = p;
+  const Point p1 = p;
   if( !pisvalid( p ) || !pnext( p ) ) return -1;
-  unsigned char ch = data[p1.line][p1.col];
-  return ch;
+  return (unsigned char)data[p1.line][p1.col];
   }
 
 
 int Basic_buffer::pngetc( Point & p ) const
   {
   if( !pnext( p ) || p == eof() ) return -1;
-  unsigned char ch = data[p.line][p.col];
-  return ch;
+  return (unsigned char)data[p.line][p.col];
   }
 
 
 int Basic_buffer::ppgetc( Point & p ) const
   {
   if( !pprev( p ) ) return -1;
-  unsigned char ch = data[p.line][p.col];
-  return ch;
+  return (unsigned char)data[p.line][p.col];
   }
 
 
@@ -365,7 +356,7 @@ Point Basic_buffer::eol( const Point & p ) const
   }
 
 
-// returns position of first non-blank character in line (eol if none)
+// return position of first non-blank character in line (eol if none)
 Point Basic_buffer::bot( const Point & p ) const
   {
   Point p1 = bol( p );
@@ -375,7 +366,7 @@ Point Basic_buffer::bot( const Point & p ) const
   }
 
 
-// returns position following last non-whitespace character in line (bol if none)
+// return position following last non-whitespace character in line (bol if none)
 Point Basic_buffer::eot( const Point & p ) const
   {
   Point p1 = eol( p );
@@ -422,10 +413,12 @@ int Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
     case '[': ch2 = ']'; forward = true; break;
     case '{': ch2 = '}'; forward = true; break;
     case '<': ch2 = '>'; forward = true; break;
+    case 0xAB: ch2 = 0xBB; forward = true; break;		// ''
     case ')': ch2 = '('; forward = false; break;
     case ']': ch2 = '['; forward = false; break;
     case '}': ch2 = '{'; forward = false; break;
     case '>': ch2 = '<'; forward = false; break;
+    case 0xBB: ch2 = 0xAB; forward = false; break;		// ''
     case '/': ch2 = '*'; forward = true; comment = true; break;
     case '*': ch2 = '/'; forward = false; comment = true; break;
     case '"':
@@ -682,3 +675,30 @@ long Basic_buffer::to_string( const Point & p1, const long size,
     }
   return text.size();
   }
+
+
+// copy text replacing special characters with escape sequences; '\n' --> "\n"
+long Basic_buffer::to_string_esc( const Point & p1, const Point & p2,
+                                  std::string & text ) const
+  {
+  text.clear();
+  if( p1 < p2 && pisvalid( p1 ) && pisvalid( p2 ) )
+    for( Point p = p1; p < p2; pnext( p ) )
+      ISO_8859::escapize( data[p.line][p.col], text );
+  return text.size();
+  }
+
+
+// copy text without whitespace characters
+long Basic_buffer::to_string_ws( const Point & p1, const Point & p2,
+                                 std::string & text ) const
+  {
+  text.clear();
+  if( p1 < p2 && pisvalid( p1 ) && pisvalid( p2 ) )
+    for( Point p = p1; p < p2; pnext( p ) )
+      {
+      const unsigned char ch = data[p.line][p.col];
+      if( !ISO_8859::isspace( ch ) ) text += ch;
+      }
+  return text.size();
+  }
