@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -177,27 +177,31 @@ bool Buffer::Options::set_lmargin( const int l )
 
 bool Buffer::Options::set_rmargin( const int r )
   {
-  if( r > lmargin_ && r < 1000 ) { rmargin_ = r; return true; }
+  if( r > lmargin_ && r < INT_MAX / 2 ) { rmargin_ = r; return true; }
+  if( r == -1 ) { rmargin_ = INT_MAX / 2 - 1; return true; }
   return false;
   }
 
 
-Buffer::Buffer( const std::string * const namep, const bool read_only )
+/* 'namep' must be 0 or "-" for stdin to be read into buffer.
+   A non-null but empty 'namep' ("") creates an empty unnamed buffer.
+*/
+Buffer::Buffer( const char * const namep, const bool read_only )
   : backuped( false ), modified_( false ), saved( false ), crlf( false )
   {
   FILE * f = 0;
-  const bool valid_namep = ( namep && namep->size() );
+  const bool valid_namep = ( namep && namep[0] );
+  const bool is_stdin = !namep || ( namep[0] == '-' && namep[1] == 0 );
   errno = 0;
-  if( ( !valid_namep || *namep == "-" ) && !isatty( fileno( stdin ) ) )
-    f = stdin;
+  if( is_stdin && !isatty( fileno( stdin ) ) ) f = stdin;
   else if( valid_namep )
     {
-    name_ = *namep;
+    name_ = namep;
     if( read_only ||
         ( !(f = std::fopen( name_.c_str(), "r+" )) && errno != ENOENT ) )
       { f = std::fopen( name_.c_str(), "r" ); options.read_only = true; }
     }
-  else return;
+  else return;				// empty unnamed buffer
 
   if( !f )
     { if( errno == ENOENT ) return; else throw Error( "Error opening file" ); }
@@ -242,7 +246,7 @@ Buffer::Buffer( const std::string * const namep, const bool read_only )
   }
 
 
-int Buffer::set_marks()		// returns the number of new marks set
+int Buffer::set_marks()		// return the number of new marks set
   {
   int c = 0;
   if( mark[0].line < 0 )
@@ -274,7 +278,7 @@ int Buffer::set_marks()		// returns the number of new marks set
      ""      x     true     error              (no name to save to)
      x     valid   false    save               (if name != *namep, save to)
    valid    ""     false    save_and_set_name  (if read_only, save to)
-   valid     x     true     save_to            (save to 'other' file)
+   valid     x     true     save_to            (save to another file)
 */
 bool Buffer::save( const std::string * namep, bool save_to,
                    const bool append, Point p1, Point p2 ) const
@@ -315,8 +319,9 @@ bool Buffer::save( const std::string * namep, bool save_to,
     }
   if( !namep ) return false;		// always save to file name '*namep'
 
-  if( !Menu::make_path( *namep ) )
-    { if( !had_name ) { name_.clear(); } throw Error( "Error creating path" ); }
+  if( !Menu::make_dirs( *namep ) )
+    { if( !had_name ) name_.clear();
+      throw Error( "Error creating intermediate directory" ); }
   if( !save_to && !saved && !append )
     {
     if( RC::editor_options().backup && !backuped )
@@ -589,7 +594,8 @@ bool Buffer::redo( Point & p )
   }
 
 
-// reformat paragraph around cursor and move cursor to ( bop.line, lmargin )
+/* Reformat paragraph around cursor.
+   If change, move cursor to ( bop.line, lmargin ) and return true. */
 bool Buffer::reformat_paragraph( Point & cursor, const bool append_change )
   {
   const Point bop = Basic_buffer::bop( cursor );	// begin of paragraph
@@ -606,22 +612,29 @@ bool Buffer::reformat_paragraph( Point & cursor, const bool append_change )
   if( bop < bot && to_cursor( bot ).col > lmargin )
     tmp.Basic_buffer::pputb( dummy, *this, bop, bot );
   tmp.options.set_lmargin( lmargin );
-  bool period2 = false;		// prev char was period in 2 spaces mode
-  bool pending_space = false;
   bool change = false;
+  bool pending_nbsp = false;
+  bool pending_space = false;
+  bool period2 = false;		// prev char was period in 2 spaces mode
   for( Point p = bot; p < eop; )
     {
     unsigned char ch = pgetc( p );
     if( std::isspace( ch ) )		// 'std' to exclude no-break space
       {
-      if( ( ch != ' ' && ch != '\n' ) ) change = true;
+      if( ( ch != ' ' && ch != '\n' ) || pending_nbsp ) change = true;
+      pending_nbsp = false;	// ignore nbsp before normal whitespace
       pending_space = true;
       }
+    else if( ch == 0xA0 )	// ignore nbsp after normal whitespace
+      { if( pending_nbsp ) change = true;
+        else if( !pending_space ) pending_nbsp = true; }
     else
       {
       if( pending_space )
         { pending_space = false; tmp.pputc( dummy, ' ' );
           if( period2 ) tmp.pputc( dummy, ' ' ); }
+      else if( pending_nbsp )
+        { pending_nbsp = false; tmp.pputc( dummy, 0xA0 ); }
       tmp.pputc( dummy, ch ); // if( dummy != tmp.eof() ) std::exit( 3 );
       period2 = ( ch == '.' && options.two_spaces );
       }
@@ -635,7 +648,7 @@ bool Buffer::reformat_paragraph( Point & cursor, const bool append_change )
       if( tmp.characters( line ) != characters( bop.line + line ) &&
           ( !tmp.blank( line ) || !blank( bop.line + line ) ) )
         { change = true; break; }
-  // detect changes that do not change total number of spaces in line
+  // detect changes that don't change total number of spaces in line
   if( !change && options.two_spaces )
     for( long line = 0; line < tmp.lines(); ++line )
       if( tmp.get_line( line ) != get_line( bop.line + line ) &&
