@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -75,7 +75,8 @@ public:
 
   // true if line is blank (invalid, empty, or containing only whitespace)
   bool blank( const long line ) const;
-  long characters( const long line ) const;
+  long characters( const unsigned long line ) const
+    { return ( line < data.size() ) ? data[line].size() : 0; }
   bool compare_lines( const long line1, const long line2 ) const
     { return ( data[line1] == data[line2] ); }
   bool empty() const { return ( data.size() == 1 && data[0].empty() ); }
@@ -132,6 +133,10 @@ public:
   Point to_pointer( const Point & cursor, Point & pcursor ) const;
   long to_string( const Point & p1, const Point & p2, std::string & text ) const;
   long to_string( const Point & p1, const long size, std::string & text ) const;
+  long to_string_esc( const Point & p1, const Point & p2,
+                      std::string & text ) const;
+  long to_string_ws( const Point & p1, const Point & p2,
+                     std::string & text ) const;
   };
 
 
@@ -163,8 +168,8 @@ class Buffer : public Basic_buffer
 
     struct Record
       {
-      Point before, after;	// Pointer position before and after the change
-      mutable bool modified;	// Status of modified flag before this record
+      Point before, after;	// pointer position before and after the change
+      mutable bool modified;	// status of modified flag before this record
       std::vector< Atom > atom_vector;
 
       Record( const Point & pb, const Point & pa, const bool m )
@@ -174,8 +179,8 @@ class Buffer : public Basic_buffer
       void delete_atoms();
       };
 
-    long index;			// Index of record to be undone
-    mutable bool appendable;	// True if last record added is appendable
+    long index;			// index of record to be undone
+    mutable bool appendable;	// true if last record added is appendable
     mutable bool force_append;
     std::vector< Record > record_vector;
 
@@ -219,8 +224,8 @@ public:
   Point mark[10];		// user bookmarks
   bool crlf;
 
-  explicit Buffer( const std::string * const namep = 0,
-                   const bool read_only = false );
+  explicit Buffer( const char * const namep = "", const bool read_only = false );
+
   int set_marks();		// set initial or extended user bookmarks
   bool save( const std::string * namep = 0, bool save_to = false,
              const bool append = false, Point p1 = Point(), Point p2 = Point() ) const;
