@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ void emergency_save()
   int i = 0;
   for( ; i < handles(); ++i )
     if( data[i].buffer().precious() ) break;
-  if( i >= handles() ) return;
+  if( i >= handles() ) return;			// nothing to save
 
   const std::string dead_name( "DEADMOE" );
   RC::editor_options().backup = false;
@@ -60,12 +60,34 @@ void emergency_save()
       std::fprintf( f, "\n*** File '%s'\n", data[i].buffer().nename().c_str() );
       std::fflush( f );
       data[i].buffer().save( &dead_name, true, true );
-      // save buffer once even if it has multiple handles
+      // save buffer only once even if it has multiple handles
       data[i].buffer().mark_as_unmodified();
       }
   std::fclose( f );
   }
 
+
+/* 'namep' must be 0 or "-" for stdin to be read into buffer.
+   A non-null but empty 'namep' ("") creates an empty unnamed buffer.
+*/
+int add_handle( const char * const namep, int i, const long line,
+                const long col, const bool read_only = false )
+  {
+  Buffer * bufp = new Buffer( namep, read_only );
+  RC::apply_all_file_options( *bufp );
+
+  if( i < 0 ) i = handles();
+  else
+    {
+    i = absolute_index( i );
+    if( i == 0 ) i = handles();
+    else { if( i <= first_ ) ++first_;
+           if( i <= last_visited_ ) ++last_visited_; }
+    }
+  data.insert( data.begin() + i, Buffer_handle( *bufp, line, col ) );
+  return circular_index( i );
+  }
+
 } // end namespace Bufhandle_vector
 
 
@@ -163,36 +185,33 @@ void Bufhandle_vector::set_first( const int ai )
   { first_ = ai % handles(); }
 
 
-int Bufhandle_vector::find_or_add_handle( const std::string * const namep,
-              int i, const long line, const long col, const bool read_only )
+/* An empty 'name' ("") creates an empty unnamed buffer.
+   'name' is std::string because it compares faster than c-style strings.
+   'prev_handles' limits comparisons to previously existing handles when
+   adding handles recursively, which decreases startup time.
+*/
+int Bufhandle_vector::find_or_add_handle( const std::string & name,
+              int i, const long line, const long col, const bool read_only,
+              const int prev_handles )
   {
-  if( namep && namep->size() )
-    for( int j = 0; j < handles(); ++j )
-      if( *namep == data[j].buffer().name() )
-        return circular_index( j );
-
-  Buffer * bufp = new Buffer( namep, read_only );
-  RC::apply_all_file_options( *bufp );
-
-  if( i < 0 ) i = handles();
-  else
+  if( name.size() )
     {
-    i = absolute_index( i );
-    if( i == 0 ) i = handles();
-    else { if( i <= first_ ) ++first_;
-           if( i <= last_visited_ ) ++last_visited_; }
+    const int n = ( prev_handles >= 0 && prev_handles <= handles() ) ?
+                  prev_handles : handles();
+    for( int j = 0; j < n; ++j )
+      if( name == data[j].buffer().name() )
+        return circular_index( j );
     }
-  data.insert( data.begin() + i, Buffer_handle( *bufp, line, col ) );
-  return circular_index( i );
+  return add_handle( name.c_str(), i, line, col, read_only );
   }
 
 
-// Add empty buffer if no files given or stdin is not a tty
+// Add unnamed buffer if no files given or stdin is not a tty
 void Bufhandle_vector::add_handle_if_pending_input_or_empty( const long line,
                                                              const long col )
   {
   if( !handles() || !isatty( fileno( stdin ) ) )
-    find_or_add_handle( 0, -1, line, col );
+    add_handle( 0, -1, line, col );
   }
 
 
