@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -389,8 +389,8 @@ int map_to_byte( const int code )
     case 0x042B: return 'Y';	// cyrillic capital letter yeru
     case 0x042C: return '\'';	// cyrillic capital letter soft sign
     case 0x042D: return 'E';	// cyrillic capital letter e
-    case 0x042E: return 0xDB;	// cyrillic capital letter yu
-    case 0x042F: return 0xC2;	// cyrillic capital letter ya
+//    case 0x042E: return 0xDB;	// cyrillic capital letter yu
+//    case 0x042F: return 0xC2;	// cyrillic capital letter ya
     case 0x0430: return 'a';	// cyrillic small letter a
     case 0x0431: return 'b';	// cyrillic small letter be
     case 0x0432: return 'v';	// cyrillic small letter ve
@@ -418,8 +418,8 @@ int map_to_byte( const int code )
     case 0x044B: return 'y';	// cyrillic small letter yeru
     case 0x044C: return '\'';	// cyrillic small letter soft sign
     case 0x044D: return 'e';	// cyrillic small letter e
-    case 0x044E: return 0xFB;	// cyrillic small letter yu
-    case 0x044F: return 0xE2;	// cyrillic small letter ya
+//    case 0x044E: return 0xFB;	// cyrillic small letter yu
+//    case 0x044F: return 0xE2;	// cyrillic small letter ya
     case 0x0450: return 0xE8;	// cyrillic small letter ie with grave
     case 0x0451: return 0xEB;	// cyrillic small letter io
     case 0x0452: return 'd';	// cyrillic small letter dje
@@ -781,7 +781,7 @@ int map_to_byte( const int code )
   }
 
 
-// Decode UCS to string of ISO-8859-15 chars. Return 0 if not found.
+// Decode UCS to string of ISO-8859-15 chars. Return 0 if code not found.
 const char * map_to_string( const int code )
   {
   switch( code )
@@ -802,9 +802,13 @@ const char * map_to_string( const int code )
     case 0x0416: return "Zh";	// cyrillic capital letter zhe
     case 0x0427: return "Ch";	// cyrillic capital letter che
     case 0x0428: return "Sh";	// cyrillic capital letter sha
+    case 0x042E: return "Yu";	// cyrillic capital letter yu
+    case 0x042F: return "Ya";	// cyrillic capital letter ya
     case 0x0436: return "zh";	// cyrillic small letter zhe
     case 0x0447: return "ch";	// cyrillic small letter che
     case 0x0448: return "sh";	// cyrillic small letter sha
+    case 0x044E: return "yu";	// cyrillic small letter yu
+    case 0x044F: return "ya";	// cyrillic small letter ya
     case 0x04D4: return "AE";	// cyrillic capital ligature a ie
     case 0x04D5: return "ae";	// cyrillic small ligature a ie
     case 0x0539: return "T'";	// armenian capital letter to
@@ -899,24 +903,6 @@ int map_to_ucs( const unsigned char ch )
   return ch;
   }
 
-
-void ucs_to_utf8( const int code, std::string & out )
-  {
-  if( code < 0 || code > 0x7FFFFFFF ) return;		// invalid code
-  if( code < 128 ) { out += code; return; }		// plain ascii
-
-  int i, mask;
-  if( code < 0x800 ) { i = 6; mask = 0xC0; }		// 110X XXXX
-  else if( code < 0x10000 ) { i = 12; mask = 0xE0; }	// 1110 XXXX
-  else if( code < 0x200000 ) { i = 18; mask = 0xF0; }	// 1111 0XXX
-  else if( code < 0x4000000 ) { i = 24; mask = 0xF8; }	// 1111 10XX
-  else { i = 30; mask = 0xFC; }				// 1111 110X
-
-  out += mask | ( code >> i );
-  while( ( i -= 6 ) >= 0 )
-    out += 0x80 | ( ( code >> i ) & 0x3F );		// 10XX XXXX
-  }
-
 } // end namespace Encoding
 
 
@@ -1111,6 +1097,24 @@ bool Encoding::ascii_encode( const std::string & in, std::string & out )
   }
 
 
+void Encoding::ucs_to_utf8( const int code, std::string & out )
+  {
+  if( code < 0 || code > 0x7FFFFFFF ) return;		// invalid code
+  if( code < 128 ) { out += code; return; }		// plain ascii
+
+  int i, mask;
+  if( code < 0x800 ) { i = 6; mask = 0xC0; }		// 110X XXXX
+  else if( code < 0x10000 ) { i = 12; mask = 0xE0; }	// 1110 XXXX
+  else if( code < 0x200000 ) { i = 18; mask = 0xF0; }	// 1111 0XXX
+  else if( code < 0x4000000 ) { i = 24; mask = 0xF8; }	// 1111 10XX
+  else { i = 30; mask = 0xFC; }				// 1111 110X
+
+  out += mask | ( code >> i );
+  while( ( i -= 6 ) >= 0 )
+    out += 0x80 | ( ( code >> i ) & 0x3F );		// 10XX XXXX
+  }
+
+
 /* Encode ISO-8859-15 encoded input string 'in' to output string 'out'.
    Return false if no encoding is needed, i.e. if the input is already
    valid UTF-8 data.
@@ -1192,8 +1196,8 @@ int Encoding::utf8_decode( const std::string & in, std::string & out,
     const int ch = map_to_byte( code );
     if( ch >= 0 ) { if( ch < 256 ) { out += ch; } i += len; continue; }
     const char * const str = map_to_string( code );
-    if( str && code >= 0xBC && code <= 0xBE &&
-        !out.empty() && std::isdigit( out[out.size()-1] ) ) out += ' ';
+    if( str && code >= 0xBC && code <= 0xBE && !out.empty() &&
+        std::isdigit( (unsigned char)out[out.size()-1] ) ) out += ' ';
     if( str ) out += str;
     else if( force ) out.append( in, i, len );
     else { if( idxp ) { *idxp = i; } return code; }
