@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -122,21 +122,44 @@ enum Code {
      SYDIAER = 0xFF	// latin small letter y with diaeresis
      };
 
+
+int parse_hex( const std::string & s, const unsigned long i, const int len )
+  {
+  if( len < 1 || len > 8 || i + len > s.size() ) return -1;
+  unsigned int result = 0;
+  for( unsigned long j = i; j < i + len; ++j )
+    {
+    const int d = ISO_8859::xvalue( s[j] );
+    if( d < 0 ) return -1; else { result <<= 4; result += d; }
+    }
+  if( result <= 0x7FFFFFFF ) return result; else return -1;
+  }
+
+
+unsigned char xdigit( const unsigned value )	// hex digit for 'value'
+  {
+  if( value <= 9 ) return '0' + value;
+  if( value <= 15 ) return 'A' + value - 10;
+  return 0;
+  }
+
 } // end namespace ISO_8859
 
 
 /* 'seq[i]' begins a escape sequence (the characters following a '\').
    Return the corresponding code and, in *lenp, the number of bytes read.
-   Return -1 if error.
+   Return -1 if unknown escape sequence, -2 if syntax error.
 */
-int ISO_8859::escape( const std::string & seq, const unsigned long i, int * lenp )
+int ISO_8859::escape( const std::string & seq, const unsigned long i,
+                      int * const lenp, const bool allow_u )
   {
-  if( i >= seq.size() ) return -1;
+  if( i >= seq.size() ) return -2;
   int len = 1;
   unsigned ch = (unsigned char)seq[i];
 
   switch( ch )
     {
+    case '\\': break;
     case 'a': ch = '\a'; break;
     case 'b': ch = '\b'; break;
     case 'e': ch = 27; break;
@@ -150,60 +173,44 @@ int ISO_8859::escape( const std::string & seq, const unsigned long i, int * lenp
       ch -= '0';
       while( len < 3 && i + len < seq.size() && ISO_8859::isodigit( seq[i+len] ) )
         { ch <<= 3; ch += seq[i+len] - '0'; ++len; }
-      if( ch > 255 ) return -1;
+      if( ch > 255 ) return -2;
       break;
+    case 'd': ch = 0;
+      while( len < 4 && i + len < seq.size() &&
+             std::isdigit( (unsigned char)seq[i+len] ) )
+        { ch *= 10; ch += seq[i+len] - '0'; ++len; }
+      if( len != 4 || ch > 255 ) return -2;
+      break;
+    case 'u':
+    case 'U': if( !allow_u ) return -2;		// syntax error
+      { len = ( ch == 'u' ) ? 5 : 9;
+        const int code = parse_hex( seq, i + 1, len - 1 );
+        if( code < 0 ) return -2; else { ch = code; break; } }
     case 'x':
-    case 'X': if( i + 2 >= seq.size() || ISO_8859::xvalue( seq[i+1] ) < 0 ||
-                  ISO_8859::xvalue( seq[i+2] ) < 0 ) return -1;
-              ch = ( ISO_8859::xvalue( seq[i+1] ) << 4 ) +
-                   ISO_8859::xvalue( seq[i+2] );
-              len = 3;
-              break;
+    case 'X': { const int code = parse_hex( seq, i + 1, 2 );
+                if( code < 0 ) return -2; else { ch = code; len = 3; break; } }
+    default: return -1;
     }
   if( lenp ) *lenp = len;
   return ch;
   }
 
 
-void ISO_8859::escapize( std::string & s )
+void ISO_8859::escapize( const unsigned char ch, std::string & out )
   {
-  for( unsigned long i = 0; i < s.size(); ++i )
+  if( !ISO_8859::iscntrl( ch ) && ch != '\\' ) { out += ch; return; }
+  out += '\\';
+  switch( ch )
     {
-    if( !ISO_8859::iscntrl( s[i] ) && s[i] != '\\' ) continue;
-    s.insert( s.begin() + i, '\\' ); ++i;
-    switch( s[i] )
-      {
-      case '\\': break;
-      case '\a': s[i] = 'a'; break;
-      case '\b': s[i] = 'b'; break;
-      case '\f': s[i] = 'f'; break;
-      case '\n': s[i] = 'n'; break;
-      case '\r': s[i] = 'r'; break;
-      case '\t': s[i] = 't'; break;
-      case '\v': s[i] = 'v'; break;
-      default:
-        {
-        s.insert( s.begin() + i, 'x' ); ++i;
-        const unsigned char ch = s[i];
-        s.insert( s.begin() + i, ISO_8859::xdigit( ch >> 4 ) ); ++i;
-        s[i] = ISO_8859::xdigit( ch & 0x0F );
-        }
-      }
-    }
-  }
-
-
-void ISO_8859::deescapize_search_string( std::string & s )
-  {
-  for( unsigned long i = 0; i + 1 < s.size(); ++i )
-    {
-    if( s[i] != '\\' ) continue;
-    int len, c = ISO_8859::escape( s, i + 1, &len );
-    if( c < 0 ) continue;
-    const unsigned char ch = s[i+1];
-    if( ( ch != c && ch != '\\' ) || len > 1 )
-      { s[i] = c; s.erase( i + 1, len ); }
-    else ++i;
+    case '\\': out += '\\'; break;
+    case '\a': out += 'a'; break;
+    case '\b': out += 'b'; break;
+    case '\f': out += 'f'; break;
+    case '\n': out += 'n'; break;
+    case '\r': out += 'r'; break;
+    case '\t': out += 't'; break;
+    case '\v': out += 'v'; break;
+    default: out += 'x'; out += xdigit( ch >> 4 ); out += xdigit( ch & 0x0F );
     }
   }
 
@@ -538,14 +545,6 @@ void ISO_8859::map_to_ascii( const unsigned char ch, std::string & out )
   }
 
 
-unsigned char ISO_8859::xdigit( const int value )
-  {
-  if( value >= 0 && value <= 9 ) return '0' + value;
-  if( value >= 10 && value <= 15 ) return 'A' + value - 10;
-  return 0;
-  }
-
-
 int ISO_8859::xvalue( const unsigned char ch )
   {
   switch( ch )
