@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
 */
 /*
    Exit status: 0 for a normal exit, 1 for environmental problems
-   (invalid flags, I/O errors, etc), 2 to indicate an invalid or
-   unreadable input file, 3 for an internal consistency error (e.g., bug)
+   (invalid command line options, I/O errors, etc), 2 to indicate an invalid
+   or unreadable input file, 3 for an internal consistency error (e.g., bug)
    which caused moe to panic.
 */
 
@@ -42,8 +42,8 @@
 namespace {
 
 const char * const     program_name = "moe";
-const char * const config_file_name = "moerc";
-const char * const     program_year = "2022";
+const char * const config_file_name = "moe.conf";
+const char * const     program_year = "2023";
 const char * invocation_name = program_name;		// default value
 
 
@@ -74,7 +74,7 @@ void show_help()
                "\nEditor options:\n"
                "  -h, --help               display this help and exit\n"
                "  -V, --version            output version information and exit\n"
-               "  -1, --orphan             put extra files in orphaned buffers\n"
+//               "  -1, --orphan             put extra files in orphaned buffers\n"
                "  -b, --backup             create backup files (default)\n"
                "  -B, --no-backup          don't create backup files\n"
                "  -e, --exit-ask           make save-and-close ask always for confirmation\n"
@@ -94,10 +94,11 @@ void show_help()
                "  -o, --read-only          make buffer read-only\n"
                "  -O, --overwrite          disable insert mode\n"
                "  -w, --word-wrap          enable word wrapping\n"
+               "  -W, --no-word-wrap       disable word wrapping\n"
                "  -2, --two-spaces         make reformat put two spaces after period\n"
                "\nExit status: 0 for a normal exit, 1 for environmental problems\n"
-               "(invalid flags, I/O errors, etc), 2 to indicate an invalid or\n"
-               "unreadable input file, 3 for an internal consistency error (e.g., bug)\n"
+               "(invalid command line options, I/O errors, etc), 2 to indicate an invalid\n"
+               "or unreadable input file, 3 for an internal consistency error (e.g., bug)\n"
                "which caused moe to panic.\n"
                "\nReport bugs to bug-moe@gnu.org\n"
                "Moe home page: http://www.gnu.org/software/moe/moe.html\n"
@@ -172,13 +173,15 @@ bool use_it_anyway( const std::string & name )
   }
 
 
-void process_config_file()
+void maybe_process_config_file()
   {
   std::fputc( '\n', stderr );
-  std::string name( RC::home_directory() );
+  std::string name;
+  const char * p = std::getenv( "XDG_CONFIG_HOME" ); if( p ) name = p;
+  else { name = RC::home_directory(); if( name.size() ) name += "/.config"; }
   if( name.size() )
     {
-    name += "/."; name += config_file_name;
+    name += '/'; name += config_file_name;
     const int retval = RC::process_rcfile( name );
     if( retval == 0 || ( retval == 2 && use_it_anyway( name ) ) ) return;
     RC::reset();
@@ -238,7 +241,7 @@ int main( const int argc, const char * const argv[] )
 
   Bufhandle_vector::init();
 
-  process_config_file();
+  maybe_process_config_file();
   retval = RC::process_options();
   if( retval == 2 )
     {
@@ -248,7 +251,7 @@ int main( const int argc, const char * const argv[] )
   if( retval ) return retval;
 
   if( !Screen::init() )
-    { RC::show_error( "Screen too small (min size 24 x 80)." ); return 1; }
+    { RC::show_error( "Screen too small (min size 25 x 80)." ); return 1; }
   set_signals();
 
   char buf[88];
