@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ int get_filename( const std::string & prompt, std::string & name,
 
 bool is_regular( const std::string & name, bool * can_readp = 0 );
 
-bool make_path( const std::string & name );
+bool make_dirs( const std::string & name );
 
 const std::string & my_basename( const std::string & name );
 
