@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -125,7 +125,7 @@ void split_filename( const std::string & name,
   for( unsigned i = name.size(); i > 0; --i )
     if( name[i-1] == '/' ) { first = i; break; }
 
-  if( first > 0 ) dirname.assign( name, 0, first );
+  if( first > 0 ) dirname.assign( name, 0, first );	// with trailing slash
   else dirname.clear();
   if( first < name.size() )
     basename.assign( name, first, name.size() );
@@ -231,9 +231,9 @@ bool Menu::file_menu( const std::string & prompt, std::string & name )
 
   tilde_expansion( name );
   split_filename( name, dirname, basename );
-  basename += '*';
+  if( basename.empty() || basename[basename.size()-1] != '*' ) basename += '*';
 
-  DIR * const dirp = opendir( dirname.size() ? dirname.c_str() : "./" );
+  DIR * const dirp = opendir( dirname.size() ? dirname.c_str() : "." );
   if( !dirp && ( errno == ENOTDIR || errno == ENOENT ) ) return true;
   bool dd = false;
   const std::string dotdot( ".." );		// put ".." always first
@@ -339,14 +339,15 @@ int Menu::get_filename( const std::string & prompt, std::string & name,
   static std::vector< std::string > file_history;
 
   file_history.push_back( name );
-  int len = Screen::get_string( prompt, file_history, abort_key, true );
-  if( len > 0 )
+  long size = Screen::get_string( prompt, file_history, abort_key, true );
+  if( size > 0 )
     {
-    if( is_regular( file_history.back() ) ) name = file_history.back();
-    else { len = -1; Screen::show_message( "Not a regular file" ); }
+    if( size > 65536 || !is_regular( file_history.back() ) )
+      { size = -1; Screen::show_message( "Not a regular file" ); }
+    else name = file_history.back();
     }
-  if( len <= 0 ) name.clear();
-  return len;
+  if( size <= 0 ) name.clear();
+  return size;
   }
 
 
@@ -368,26 +369,25 @@ bool Menu::is_regular( const std::string & name, bool * can_readp )
   }
 
 
-bool Menu::make_path( const std::string & name )
+bool Menu::make_dirs( const std::string & name )
   {
   std::string dirname, basename;
   split_filename( name, dirname, basename );
   if( basename.empty() || basename == "." || basename == ".." ) return false;
 
-  unsigned index = 0;
-  while( index < dirname.size() )
+  for( unsigned i = 0; i < dirname.size(); )
     {
-    while( index < dirname.size() && dirname[index] == '/' ) ++index;
-    unsigned first = index;
-    while( index < dirname.size() && dirname[index] != '/' ) ++index;
-    if( first < index )
+    while( i < dirname.size() && dirname[i] == '/' ) ++i;
+    const unsigned first = i;
+    while( i < dirname.size() && dirname[i] != '/' ) ++i;
+    if( first < i )
       {
-      std::string partial( dirname, 0, index );
+      std::string partial( dirname, 0, i );
       struct stat st;
       if( stat( partial.c_str(), &st ) == 0 )
         { if( !S_ISDIR( st.st_mode ) ) return false; }
-      else if( mkdir( partial.c_str(), S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH ) )
-        return false;
+      else if( mkdir( partial.c_str(), S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH |
+                      S_IXOTH ) != 0 && errno != EEXIST ) return false;
       }
     }
   return true;
