@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ const char * help_line[] = {
   " @bGO TO@b                  @bSEARCH@b                      @bMISC@b",
   " ^[U Beg of file        ^F  Find                    Ins  Insert/Overwrite",
   " ^[V End of file        ^G  Find next               Tab  File name completion",
-  " ^[L Line               ^W  Find word               ^A 1-90AB Alternative @bFn@b",
+  " ^[L Line               ^W  Find word               ^A 1-90AB  Alternative @bFn@b",
   " ^[C Column             ^OF Toggle direction        ^B   Reformat paragraph",
   " ^[O Offset             ^QU update (C)              ^P   Control char",
   " ^[B Beg of block                                   ^Y   Delete line",
@@ -47,9 +47,9 @@ const char * help_line[] = {
   "                        ^Sn  Show LE multibyte      ^Q@bF2@b Save named buffers",
   " @bSCROLL@b                 ^S@bFn@b Show BE multibyte",
   " ^[W Up                 ^SB  Show clock blinking    @bBOOKMARKS@b",
-  " ^[Z Down               ^SC  Show character code    ^[ 0-9 Goto",
-  " ^[A Left               ^SG  Show global status     ^K 0-9 Set",
-  " ^[S Right              ^SP  Show line position     ^KE    Extend",
+  " ^[Z Down               ^SC  Show character code    ^[ 0-9  Goto",
+  " ^[A Left               ^SG  Show global status     ^K 0-9  Set",
+  " ^[S Right              ^SP  Show line position     ^KE     Extend",
   "                        ^SV  Show version",
   "",
   " Help Screen: Block Commands     PAGE_DN next/PAGE_UP prev",
@@ -79,12 +79,12 @@ const char * help_line[] = {
   "",
   " Help Screen: Search             PAGE_DN next/PAGE_UP prev",
   " ",
-  " Escape sequences for special characters:",
+  " Escape sequences for special characters (search or replace (except \\ooo)):",
   "   \\a  BEL (alert)       \\n  LF (newline)           \\\\   '\\' (backslash)",
-  "   \\b  BS  (backspace)   \\r  CR (carriage return)   \\ooo  octal code 'ooo'",
-  "   \\e  ESC (escape)      \\t  HT (tab)               \\xhh  hexadecimal code 'hh'",
-  "   \\f  FF  (form feed)   \\v  VT (vertical tab)",
-  " ",
+  "   \\b  BS  (backspace)   \\r  CR (carriage return)   \\dddd decimal code 'ddd'",
+  "   \\e  ESC (escape)      \\t  HT (tab)               \\ooo  octal code 'ooo'",
+  "   \\f  FF  (form feed)   \\v  VT (vertical tab)      \\xhh  hexadecimal code 'hh'",
+  "                                                    \\uhhhh \\Uhhhhhhhh  UTF-8",
   " Special search sequences:",
   "   \\?      matches any single char       \\^  \\$  matches beginning/end of line",
   "   \\*      matches 0 or more chars       \\<  \\>  matches beginning/end of word",
@@ -97,10 +97,11 @@ const char * help_line[] = {
   "   \\+<ch>  matches 0 or more of <ch> (also \\[..], \\w, or \\W)",
   " ",
   " Notes",
-  "     Ranges of characters may be specified in sets with 'A-Z'",
-  "     '-' may be specified in sets by placing it first or last",
-  "     ']' may be specified in sets by placing it first",
-  "     \\+ may precede itself any number of times, matching N-1 chars",
+  "   Ranges of characters may be specified in sets with 'A-Z'",
+  "   '-' may be specified in sets by placing it first or last",
+  "   ']' may be specified in sets by placing it first",
+  "   \\u and \\U may not be specified in sets nor after \\+",
+  "   \\+ may precede itself any number of times, matching N-1 chars",
   "",
   " Help Screen: Replace            PAGE_DN next/PAGE_UP prev",
   " ",
@@ -109,6 +110,10 @@ const char * help_line[] = {
   "   \\c      same as \\& but matched text is capitalized",
   "   \\l  \\u  same as \\& but matched text is lowercased / uppercased",
   "   \\0 - 9  replaced with text which matched the Nth special search sequence",
+  " ",
+  " Notes",
+  "   \\u0 may be specified as \\u\\x30 to avoid ambiguity with \\uhhhh",
+  "   Octal escape sequences can't be used in the replacement string",
   "",
   " Help Screen: Character Set      PAGE_DN next/PAGE_UP prev",
   "  \x20   \x21   \x22   \x23   \x24   \x25   \x26   \x27   \x28   \x29   \x2A   \x2B   \x2C   \x2D   \x2E   \x2F",
