@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,8 +44,8 @@
 #endif
 
 // order of application of file options by place of specification:
-// 1 options specified in rc file as 'default file options'.
-// 2 options specified in rc file as 'file name dependent options'.
+// 1 options specified in moe.conf file as 'default file options'.
+// 2 options specified in moe.conf file as 'file name dependent options'.
 // 3 options specified in command line as 'default file options'.
 // 4 options specified in command line as 'file options'.
 // (4 is not applied to files opened during the editing sesion).
@@ -84,7 +84,7 @@ struct Regex_options
 
 Options editor_options_;
 File_options default_file_options;
-File_options moerc_file_options;
+File_options moeconf_file_options;
 std::vector< Regex_options > regex_options_vector;
 
 
@@ -95,10 +95,10 @@ enum Optcode
   max_windows = 'm', orphan_extra= '1', rectangle_mode = 'x',
   search_wrap = 's', smart_home = 'H',
   auto_indent = 'a', lmargin = 'l', rmargin = 'r', overwrite = 'O',
-  read_only = 'o', two_spaces = '2', word_wrap = 'w',
+  read_only = 'o', two_spaces = '2', word_wrap = 'w', no_word_wrap = 'W',
   no_auto_unmark = 256, /*no_beep,*/ no_exit_ask, no_ignore_case,
   no_rectangle_mode, no_search_wrap, no_smart_home,
-  no_auto_indent, no_overwrite, no_read_only, no_two_spaces, no_word_wrap
+  no_auto_indent, no_overwrite, no_read_only, no_two_spaces
   };
 
 const Arg_parser::Option options[] =
@@ -167,7 +167,7 @@ bool set_option_from_code( const Arg_parser & parser, const int argind,
   {
   const int code = parser.code( argind );
   const char * const parsed_name = parser.parsed_name( argind ).c_str();
-  const std::string & arg = parser.argument( argind );
+  const char * const arg = parser.argument( argind ).c_str();
   if( !allow_editor_options && is_editor_option( code ) )
     { std::fprintf( stderr, "%serror: Option '%s' not allowed here.\n",
                     msg, parsed_name ); return false; }
@@ -239,16 +239,16 @@ bool set_option_from_code( const Arg_parser & parser, const int argind,
   }
 
 
-int parse_line_col( const std::string & s, long & line, long & col )
+int parse_line_col( const char * const ptr, long & line, long & col )
   {
-  if( s == "+" ) { line = col = LONG_MAX - 1; return 1; }
-  int c = parse_long( s, line );
+  if( ptr[0] == '+' && ptr[1] == 0 ) { line = col = LONG_MAX - 1; return 1; }
+  int c = parse_long( ptr, line );
   if( c <= 0 || --line < 0 ) return 0;
-  if( c < (int)s.size() )
+  if( ptr[c] )
     {
-    int i = parse_long( s.c_str() + c + 1, col );
-    if( i <= 0 || --col < 0 || s[c] != ',' ) return 0;
-    c += i + 1;
+    const int c2 = parse_long( ptr + c + 1, col );
+    if( c2 <= 0 || --col < 0 || ptr[c] != ',' ) return 0;
+    c += c2 + 1;
     }
   else col = 0;
   return c;
@@ -303,9 +303,10 @@ bool unwanted_name( const std::string & name )
   const char * const ext2[] = { "gz", "lz", "mo", "po", "ps", "so", "xz", 0 };
   const char * const ext3[] = { "bmp", "bz2", "deb", "dll", "dvi", "elf",
     "exe", "gif", "gmo", "ico", "jar", "jpg", "obj", "pbm", "pdf", "pgm",
-    "png", "ppm", "rpm", "svg", "tar", "tgz", "tif", "tlz", "txz", "wav",
-    "xcf", "zip", 0 };
-  const char * const ext4[] = { "info", "jpeg", "rpms", "svgz", 0 };
+    "png", "pnm", "ppm", "rpm", "svg", "tar", "tgz", "tif", "tlz", "txz",
+    "wav", "xcf", "zip", "zst", 0 };
+  const char * const ext4[] = { "info", "jpeg", "lzma", "rpms", "svgz",
+    "tiff", "tzst", 0 };
   const char * const ext5[] = { "class", 0 };
   const char * const * ext[5] = { ext1, ext2, ext3, ext4, ext5 };
 
@@ -323,8 +324,9 @@ bool unwanted_name( const std::string & name )
   }
 
 
-// Depth first. Files first in each directory. Load regular files only.
-// Ignore backups '*~', unwanted names, and symbolic links.
+/* Depth first. Files first in each directory. Load regular files only.
+   Ignore backups '*~', unwanted names, and symbolic links.
+*/
 bool add_handles_recursively( std::string name, const long line, const long col,
                               const File_options & file_options,
                               bool & first_post, bool & read_error_found )
@@ -333,12 +335,13 @@ bool add_handles_recursively( std::string name, const long line, const long col,
     name.resize( name.size() - 1 );		// reject "/", "//", etc.
   if( name.empty() ) return false;
   while( name.size() > 2 && name[0] == '.' && name[1] == '/' )
-    name.erase( 0, 2 );				// remove leading ./
+    name.erase( 0, 2 );				// remove leading "./"
   struct stat st;
   if( stat( name.c_str(), &st ) != 0 || !S_ISDIR( st.st_mode ) )
     return false;
   std::list< std::string > dirnames;		// only directory names here
   dirnames.push_back( name );
+  const int prev_handles = Bufhandle_vector::handles();
   while( !dirnames.empty() )
     {
     const std::string dirname = dirnames.front();
@@ -359,16 +362,16 @@ bool add_handles_recursively( std::string name, const long line, const long col,
     if( name_vector.empty() ) continue;
     if( name_vector.size() > 1 )
       std::sort( name_vector.begin(), name_vector.end() );
-    std::list< std::string > tmp_list;		// subdirs temporary list
+    std::list< std::string > subdir_list;	// subdirs temporary list
     for( unsigned i = 0; i < name_vector.size(); )
       {
-      const unsigned dot = name_vector[i].rfind( '.' );
-      const unsigned slash = name_vector[i].rfind( '/' );
-      const unsigned size = name_vector[i].size();
-      const unsigned j = i++;
+      const unsigned long dot = name_vector[i].rfind( '.' );
+      const unsigned long slash = name_vector[i].rfind( '/' );
+      const unsigned long size = name_vector[i].size();
+      const unsigned j = i++;	// names differing only in the extension
       if( dot < size - 1 && dot > 0 && ( slash >= size || slash + 1 < dot ) &&
-          ( !std::isdigit( name_vector[j][dot-1] ) ||
-            !std::isdigit( name_vector[j][dot+1] ) ) )
+          ( !std::isdigit( (unsigned char)name_vector[j][dot-1] ) ||
+            !std::isdigit( (unsigned char)name_vector[j][dot+1] ) ) )
         while( i < name_vector.size() &&
                name_vector[j].compare( 0, dot + 1, name_vector[i],
                                        0, dot + 1 ) == 0 &&
@@ -376,12 +379,12 @@ bool add_handles_recursively( std::string name, const long line, const long col,
       for( unsigned k = i; k > j; --k )		// reverse sort by extension
         {
         if( lstat( name_vector[k-1].c_str(), &st ) != 0 ) continue;
-        if( S_ISDIR( st.st_mode ) ) tmp_list.push_back( name_vector[k-1] );
+        if( S_ISDIR( st.st_mode ) ) subdir_list.push_back( name_vector[k-1] );
         else if( S_ISREG( st.st_mode ) )
           {
           try {
             const int h = Bufhandle_vector::find_or_add_handle(
-                            &name_vector[k-1], -1, line, col );
+                      name_vector[k-1], -1, line, col, false, prev_handles );
             apply_file_options( Bufhandle_vector::handle( h ).buffer(),
                                 file_options );
             }
@@ -395,7 +398,7 @@ bool add_handles_recursively( std::string name, const long line, const long col,
           }
         }
       }
-    dirnames.splice( dirnames.begin(), tmp_list );	// prepend subdirs
+    dirnames.splice( dirnames.begin(), subdir_list );	// prepend subdirs
     }
   return true;
   }
@@ -408,52 +411,50 @@ RC::Options & RC::editor_options() { return editor_options_; }
 
 void RC::apply_all_file_options( Buffer & buffer )
   {
-  apply_file_options( buffer, moerc_file_options );
+  apply_file_options( buffer, moeconf_file_options );
   apply_regex_options( buffer );
   apply_file_options( buffer, default_file_options );
   }
 
 
 // Return true if first non-whitespace character is '+' or '-'.
-bool RC::issigned( const std::string & s )
+bool RC::issigned( const char * ptr )
   {
-  unsigned i = 0;
-  while( i < s.size() && ISO_8859::isspace( s[i] ) ) ++i;	// strip spaces
-  return ( i < s.size() && ( s[i] == '+' || s[i] == '-' ) );
+  while( *ptr && ISO_8859::isspace( *ptr ) ) ++ptr;	// strip spaces
+  return *ptr == '+' || *ptr == '-';
   }
 
 
 // Return the number of bytes read, or 0 if error.
-int RC::parse_int( const std::string & s, int & result )
+int RC::parse_int( const char * const ptr, int & result )
   {
   long tmp;
-  const int c = parse_long( s, tmp );
+  const int c = parse_long( ptr, tmp );
   if( !c || tmp < -INT_MAX || tmp > INT_MAX ) return 0;
   result = tmp;
   return c;
   }
 
 // Return the number of bytes read, or 0 if error.
-int RC::parse_long( const std::string & s, long & result )
+int RC::parse_long( const char * const ptr, long & result )
   {
-  if( s.empty() ) return 0;
-  const char * const str = s.c_str();
+  if( !ptr || !ptr[0] ) return 0;
   char * tail;
   errno = 0;
-  const long tmp = std::strtol( str, &tail, 0 );
-  if( errno || tail == str || tmp < -LONG_MAX ) return 0;
+  const long tmp = std::strtol( ptr, &tail, 0 );
+  if( errno || tail == ptr || tmp < -LONG_MAX ) return 0;
   result = tmp;
-  const int c = tail - str;
+  const int c = tail - ptr;
   return c;
   }
 
 // Return the number of bytes read, or 0 if error.
-int RC::parse_relative_long( const std::string & s, long & result )
+int RC::parse_relative_long( const char * const ptr, long & result )
   {
   long tmp = 0;
-  const int c = parse_long( s, tmp );
+  const int c = parse_long( ptr, tmp );
   if( !c ) return 0;
-  if( !issigned( s ) ) result = tmp;
+  if( !issigned( ptr ) ) result = tmp;
   else if( tmp > 0 )
     {
     if( LONG_MAX - tmp < result ) result = LONG_MAX;
@@ -500,7 +501,7 @@ int RC::process_rcfile( const std::string & name )
       retval = 2;
       }
     if( len == 0 || ISO_8859::isspace( buf[0] ) ) continue;
-    if( buf[0] != '-' )		// Create new file name dependent options
+    if( buf[0] != '-' )		// create new file name dependent options
       {
       const char * const regex = std::strtok( buf, delimiters );
       if( regex_status != 1 )
@@ -508,7 +509,7 @@ int RC::process_rcfile( const std::string & name )
       regex_options_vector.back().regex_vector.push_back( regex );
       regex_status = 1;
       }
-    else			// Set an option
+    else			// set an option
       {
       const char * const opt = std::strtok( buf, delimiters );
       const char * const arg = std::strtok( 0, delimiters );
@@ -520,16 +521,13 @@ int RC::process_rcfile( const std::string & name )
         { std::fprintf( stderr, "%s%s\n", msg, parser.error().c_str() );
           retval = 2; }
       else if( regex_status == 0 )
-        { if( !set_option_from_code( parser, 0,
-                                     moerc_file_options, msg, true ) )
-            retval = 2; }
+        { if( !set_option_from_code( parser, 0, moeconf_file_options,
+                                     msg, true ) ) retval = 2; }
       else
-        {
-        if( !set_option_from_code( parser, 0,
-                                   regex_options_vector.back().file_options,
-                                   msg ) ) retval = 2;
-        regex_status = 2;
-        }
+        { if( !set_option_from_code( parser, 0,
+                                     regex_options_vector.back().file_options,
+                                     msg ) ) retval = 2;
+          regex_status = 2; }
       }
     }
   std::fclose( f );
@@ -546,7 +544,7 @@ int RC::process_options()
 
   int argind = 0;
   for( ; argind < parserp->arguments(); ++argind )
-    {						// Process "global" options
+    {						// process "global" options
     if( !parserp->code( argind ) ) break;	// line number or file name
     if( !set_option_from_code( *parserp, argind,
                                default_file_options, "\n", true ) ) return 1;
@@ -557,11 +555,11 @@ int RC::process_options()
   bool read_error_found = false;
   bool unsupported_found = false;
   bool stdin_used = false;
-  while( argind < parserp->arguments() )  // Process files and file options
+  while( argind < parserp->arguments() )  // process files and file options
     {
-    if( parserp->argument( argind )[0] == '+' )		// line number
+    if( parserp->argument( argind ).c_str()[0] == '+' )		// line number
       {
-      if( !parse_line_col( parserp->argument( argind ), line, col ) )
+      if( !parse_line_col( parserp->argument( argind ).c_str(), line, col ) )
         {
         if( first_post ) std::fputc( '\n', stderr );
         show_error( "Bad number of line or column." );
@@ -579,7 +577,7 @@ int RC::process_options()
     Menu::tilde_expansion( name );
     File_options file_options;
     for( ; argind < parserp->arguments(); ++argind )
-      {						// Process file options
+      {						// process file options
       if( !parserp->code( argind ) ) break;	// line number or file name
       if( !set_option_from_code( *parserp, argind, file_options,
                                  first_post ? "\n" : "" ) ) return 1;
@@ -588,10 +586,11 @@ int RC::process_options()
     const bool is_stdin = ( name == "-" );
     if( is_stdin && ( stdin_used || isatty( fileno( stdin ) ) ) )
       ;		// ignore stdin if is repeated or is a terminal
-    else if( Menu::is_regular( name, &can_read ) || can_read )
+    else if( is_stdin || name.empty() ||
+             Menu::is_regular( name, &can_read ) || can_read )
       {
       try {
-        const int i = Bufhandle_vector::find_or_add_handle( &name, -1, line,
+        const int i = Bufhandle_vector::find_or_add_handle( name, -1, line,
                                                             col, can_read );
         apply_file_options( Bufhandle_vector::handle( i ).buffer(), file_options );
         }
@@ -611,7 +610,7 @@ int RC::process_options()
       }
     if( is_stdin ) stdin_used = true;
     }
-  // Add empty buffer if no files given or stdin is not a tty
+  // add empty buffer if no files given or stdin is not a tty
   try { Bufhandle_vector::add_handle_if_pending_input_or_empty( line, col ); }
   catch( Buffer::Error & e )
     {
@@ -650,6 +649,6 @@ void RC::reset()
   {
   editor_options_.reset();
   default_file_options.reset();
-  moerc_file_options.reset();
+  moeconf_file_options.reset();
   regex_options_vector.clear();
   }
