@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ public:
     : indent_step_( 2 ), keep_lines_( -1 ), max_windows_( 0 ),
     show_code_lpos( 0 ),
     auto_unmark( false ), backup( true ), /*beep( false ),*/ exit_ask( false ),
-    ignore_case( false ), orphan_extra( false ), rectangle_mode( false ),
+    ignore_case( false ), orphan_extra( true ), rectangle_mode( false ),
     search_wrap( false ), smart_home( true ) {}
 
   int indent_step() const { return indent_step_; }
@@ -46,17 +46,17 @@ public:
   };
 
 
-Options & editor_options();			// Global editor options
+Options & editor_options();			// global editor options
 
 void apply_all_file_options( Buffer & buffer );
 
 // Return true if first non-whitespace character is '+' or '-'.
-bool issigned( const std::string & s );
+bool issigned( const char * ptr );
 
 // Return the number of bytes read, or 0 if error.
-int parse_int( const std::string & s, int & result );
-int parse_long( const std::string & s, long & result );
-int parse_relative_long( const std::string & s, long & result );
+int parse_int( const char * const ptr, int & result );
+int parse_long( const char * const ptr, long & result );
+int parse_relative_long( const char * const ptr, long & result );
 
 // Return 0 if success, 1 if file not found, 2 if syntax error.
 int process_rcfile( const std::string & name );
