@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -47,32 +47,29 @@ Set::Set( const std::string & regex, const unsigned i0 )
   unsigned i = i0 + 1;
   bool fail = true;
   if( regex[i] == '^' ) { ++i; in = false; }
-  if( regex[i] == ']' ) data[regex[i++]] = true;
+  const unsigned i1 = i;			// position of first char
 
   for( ; i < regex.size(); ++i )
     {
     unsigned char ch1 = regex[i];
-    if( ch1 == ']' ) { ++i; fail = false; break; }
+    if( ch1 == ']' && i > i1 ) { ++i; fail = false; break; }
     if( ch1 == '\\' )
       {
-      int len, cht = ISO_8859::escape( regex, i + 1, &len );
-      if( cht < 0 ) break;
-      ch1 = cht; i += len;
+      int len, code = ISO_8859::escape( regex, i + 1, &len );
+      if( code < 0 ) break;
+      ch1 = code; i += len;
       }
     if( i + 2 >= regex.size() || regex[i+1] != '-' || regex[i+2] == ']' )
-      data[ch1] = true;
-    else					// range
+      { data[ch1] = true; continue; }		// single char
+    i += 2;					// range
+    unsigned char ch2 = regex[i];
+    if( ch2 == '\\' )
       {
-      i += 2;
-      unsigned char ch2 = regex[i];
-      if( ch2 == '\\' )
-        {
-        int len, cht = ISO_8859::escape( regex, i + 1, &len );
-        if( cht < 0 ) break;
-        ch2 = cht; i += len;
-        }
-      for( int c = ch1; c <= ch2; ++c ) data[c] = true;
+      int len, code = ISO_8859::escape( regex, i + 1, &len );
+      if( code < 0 ) break;
+      ch2 = code; i += len;
       }
+    for( int c = ch1; c <= ch2; ++c ) data[c] = true;
     }
   if( !fail ) parsed_len_ = i - i0;
   }
@@ -86,24 +83,24 @@ inline int case_plus( const Basic_buffer & buffer, Point & p,
                       const std::string & regex, unsigned & i,
                       std::vector< std::string > & pieces, const bool icase )
   {
-  if( ++i >= regex.size() ) return 2;
+  if( i >= regex.size() ) return 2;
   const int ps = pieces.size();
-  std::string piece;		// Find shortest matching sequence
-  int counter = 0;		// Minimum number of matches required
+  std::string piece;		// find shortest matching sequence
+  int counter = 0;		// minimum number of matches required
 
   while( i + 1 < regex.size() && regex[i] == '\\' && regex[i+1] == '+' )
     { ++counter; i += 2; }
 
-  unsigned char ch1 = regex[i];
+  const unsigned char ch1 = regex[i];
   if( ch1 == '\\' )
     {
-    if( i + 1 >= regex.size() ) return 2;
-    const unsigned char chr = regex[i+1];
+    if( ++i >= regex.size() ) return 2;
+    const unsigned char chr = regex[i];
     if( chr == '[' )
       {
-      Set set( regex, i + 1 );		// Set of characters to match
+      Set set( regex, i );		// set of characters to match
       if( !set.parsed_len() ) return 2;
-      i += set.parsed_len();
+      i += set.parsed_len() - 1;
       for( int j = 0; ; ++j )
         {
         if( j >= counter )
@@ -120,7 +117,6 @@ inline int case_plus( const Basic_buffer & buffer, Point & p,
       }
     if( chr == 'w' || chr == 'W' )
       {
-      ++i;
       for( int j = 0; ; ++j )
         {
         if( j >= counter )
@@ -135,9 +131,7 @@ inline int case_plus( const Basic_buffer & buffer, Point & p,
       pieces.insert( pieces.begin() + ps, piece ); i = regex.size();
       return 0;
       }
-    int len, cht = ISO_8859::escape( regex, i + 1, &len );
-    if( cht < 0 ) return 2;
-    ch1 = cht; i += len;
+    if( chr != '\\' ) return 2;
     }
   for( int j = 0; ; ++j )
     {
@@ -196,7 +190,7 @@ int match_regex( const Basic_buffer & buffer, Point & p,
                 if( ch2 < 0 ) { retval = 1; break; }
                 pieces.push_back( std::string( 1, ch2 ) );
                 } break;
-      case '*': {			// Find shortest matching sequence
+      case '*': {			// find shortest matching sequence
                 const int ps = pieces.size();
                 std::string piece;
                 while( true )
@@ -237,16 +231,16 @@ int match_regex( const Basic_buffer & buffer, Point & p,
                 i = regex.size();
                 } break;
       case '[': {
-                Set set( regex, i );	// Set of characters to match
+                Set set( regex, i );	// set of characters to match
                 if( !set.parsed_len() ) { retval = 2; break; }
                 const int ch2 = buffer.pgetc( p );
                 if( ch2 < 0 || !set.includes( ch2 ) ) { retval = 1; break; }
                 i += set.parsed_len() - 1;
                 pieces.push_back( std::string( 1, ch2 ) );
                 } break;
-      case '+': retval = case_plus( buffer, p, regex, i, pieces, icase );
+      case '+': retval = case_plus( buffer, p, regex, ++i, pieces, icase );
                 break;
-      case 'S': {			// Find longest matching sequence
+      case 'S': {			// find longest matching sequence
                 std::string piece;
                 while( true )
                   {
@@ -258,7 +252,7 @@ int match_regex( const Basic_buffer & buffer, Point & p,
                 if( piece.size() ) pieces.push_back( piece );
                 else retval = 1;
                 } break;
-      case 's': {			// Find shortest matching sequence
+      case 's': {			// find shortest matching sequence
                 const int ps = pieces.size();
                 std::string piece;
                 while( true )
@@ -272,7 +266,7 @@ int match_regex( const Basic_buffer & buffer, Point & p,
                 if( retval == 0 )
                   { pieces.insert( pieces.begin() + ps, piece ); i = regex.size(); }
                 } break;
-      case 'T': {			// Find longest matching sequence
+      case 'T': {			// find longest matching sequence
                 std::string piece;
                 while( true )
                   {
@@ -292,16 +286,11 @@ int match_regex( const Basic_buffer & buffer, Point & p,
                   pieces.push_back( std::string( 1, ch2 ) );
                 else retval = 1;
                 } break;
-      default: {
-               int len, cht = ISO_8859::escape( regex, i, &len );
-               if( cht < 0 ) { retval = 2; break; }
-               ch1 = cht;
-               const int ch2 = buffer.pgetc( p );
-               if( ch2 < 0 ||
-                   ( icase && ISO_8859::tolower( ch1 ) != ISO_8859::tolower( ch2 ) ) ||
-                   ( !icase && ch1 != ch2 ) ) retval = 1;
-               else i += len - 1;
-               }
+      case '\\': {
+                 const int ch2 = buffer.pgetc( p );
+                 if( ch2 < 0 || ch1 != ch2 ) retval = 1;
+                 } break;
+      default: retval = 2;
       }
     }
   if( retval )
@@ -316,7 +305,8 @@ int match_regex( const Basic_buffer & buffer, Point & p,
 /* Search 'buffer' from 'p1' ('p1-1' if backward) for 'regex'.
    If found, set 'p1' at the first char of, and 'p2' after the end of, the
    substring found, and return in 'pieces' the text matching every '\*',
-   '\?', etc, from the regular expresion.
+   '\?', etc, from the regular expresion. 'regex' must not contain escape
+   sequences for special characters except backslash "\\".
    Return 0 if match, 1 if no match, 2 if trouble.
 */
 int Regex::find( const Basic_buffer & buffer, Point & p1, Point & p2,
@@ -326,25 +316,25 @@ int Regex::find( const Basic_buffer & buffer, Point & p1, Point & p2,
   if( regex.empty() ) return 0;
   if( pieces.size() ) pieces.clear();
 
-  const unsigned bs = regex.find('\\');
+  const unsigned long bs = regex.find('\\');
   if( bs > 0 )
     {
-    const std::string header( regex, 0, bs );
-    const bool same_size = ( header.size() == regex.size() );
+    const std::string prefix( regex, 0, bs );	// ordinary characters
+    const bool fixed_string = ( prefix.size() == regex.size() );
     if( backward ) while( true )
       {
-      if( !buffer.rfind_text( p1, header, icase ) ) return 1;
-      p2 = p1;
-      if( same_size ) { buffer.pseek( p2, header.size() ); return 0; }
+      if( !buffer.rfind_text( p1, prefix, icase ) ) return 1;
+      p2 = p1;						// prefix found
+      if( fixed_string ) { buffer.pseek( p2, prefix.size() ); return 0; }
       const int ret = match_regex( buffer, p2, regex, 0, pieces, icase );
       if( ret != 1 ) return ret;
       if( !buffer.pprev( p1 ) ) return 1;
       }
     else while( true )
       {
-      if( !buffer.find_text( p1, header, icase ) ) return 1;
-      p2 = p1; buffer.pseek( p1, -header.size() );
-      if( same_size ) return 0; else p2 = p1;
+      if( !buffer.find_text( p1, prefix, icase ) ) return 1;
+      p2 = p1; buffer.pseek( p1, -prefix.size() );	// prefix found
+      if( fixed_string ) return 0; else p2 = p1;
       const int ret = match_regex( buffer, p2, regex, 0, pieces, icase );
       if( ret != 1 ) return ret;
       if( !buffer.pnext( p1 ) || p1 == buffer.eof() ) return 1;
@@ -368,7 +358,8 @@ int Regex::find( const Basic_buffer & buffer, Point & p1, Point & p2,
 
 
 /* Delete block at [p1,p2) and replace it with the text produced from
-   'regex' and 'pieces'.
+   'regex' and 'pieces'. 'regex' must not contain escape sequences for
+   special characters except backslash "\\".
    If there is no error, return true and set 'p2' after the newly
    inserted text.
 */
@@ -376,20 +367,21 @@ bool Regex::replace( Buffer & buffer, const Point & p1, Point & p2,
                      const std::string & regex,
                      const std::vector< std::string > & pieces )
   {
-  const Basic_buffer matched( buffer, p1, p2 );
   Basic_buffer tmp;
   Point p = tmp.bof();
   bool fail = false;
 
   for( unsigned i = 0; i < regex.size() && !fail; ++i )
     {
-    unsigned char ch1 = regex[i];
-    if( ch1 != '\\' )			// normal (not escaped) char
-      { tmp.pputc( p, ch1 ); continue; }
+    const unsigned char ch0 = regex[i];
+    if( ch0 != '\\' )			// normal (not escaped) char
+      { tmp.pputc( p, ch0 ); continue; }
 
     if( ++i >= regex.size() ) { fail = true; break; }
-    ch1 = regex[i];	// ch1 is first (maybe only) of a escape secuence
-    if( std::isdigit( ch1 ) )
+    // ch1 is first (maybe only) char of a escape secuence
+    const unsigned char ch1 = regex[i];
+    if( ch1 == '\\' ) tmp.pputc( p, ch1 );
+    else if( std::isdigit( ch1 ) )
       {
       const unsigned n = ch1 - '0';
       if( n < pieces.size() )
@@ -397,23 +389,19 @@ bool Regex::replace( Buffer & buffer, const Point & p1, Point & p2,
           tmp.pputc( p, pieces[n][j] );
       else fail = true;
       }
-    else if( ch1 == '&' || ch1 == 'c' || ch1 == 'l' || ch1 == 'u' )
+    else if( ch1 == '&' || ch1 == 'c' || ch1 == 'l' ||
+             ( ch1 == 'u' &&
+               ( i + 1 >= regex.size() || ISO_8859::xvalue( regex[i+1] ) < 0 ) ) )
       {
-      if( !matched.empty() )
+      const Point begin = p;
+      if( tmp.pputb( p, buffer, p1, p2 ) && begin < p )
         {
-        const Point begin = p;
-        tmp.pputb( p, matched, matched.bof(), matched.eof() );
         if( ch1 == 'c' ) tmp.capitalize( begin, p );
         else if( ch1 == 'l' ) tmp.to_lowercase( begin, p );
         else if( ch1 == 'u' ) tmp.to_uppercase( begin, p );
         }
       }
-    else
-      {
-      int len, cht = ISO_8859::escape( regex, i, &len );
-      if( cht >= 0 ) { tmp.pputc( p, cht ); i += len - 1; }
-      else fail = true;
-      }
+    else fail = true;
     }
   if( !fail && !buffer.replace( p1, p2, tmp, tmp.bof(), tmp.eof() ) )
     fail = true;
@@ -428,10 +416,11 @@ bool Regex::match_filename( const std::string & regex, const std::string & name,
   {
   for( ; i1 < regex.size(); ++i1 )
     {
-    unsigned char ch1 = regex[i1];
+    const unsigned char ch1 = regex[i1];
     if( ch1 == '*' )
       {
       if( i1 + 1 >= regex.size() ) return true;		// tail match
+      if( regex[i1+1] == '*' ) continue;		// coalesce asterisks
       for( ; i2 < name.size(); ++i2 )
         if( match_filename( regex, name, i1 + 1, i2 ) ) return true;
       return false;
@@ -441,19 +430,18 @@ bool Regex::match_filename( const std::string & regex, const std::string & name,
       {
       case '?': ++i2; break;
       case '[': {
-                Set set( regex, i1 );		// Set of characters to match
+                Set set( regex, i1 );		// set of characters to match
                 if( !set.includes( name[i2++] ) ) return false;
                 i1 += set.parsed_len() - 1;
                 } break;
-      default: if( ch1 == '\\' )
-                 {
-                 int len, cht = ISO_8859::escape( regex, i1 + 1, &len );
-                 if( cht < 0 ) return false;
-                 ch1 = cht; i1 += len;
-                 }
-               if( ch1 != (unsigned char)name[i2++] ) return false;
+      case '\\': {
+                int len, code = ISO_8859::escape( regex, i1 + 1, &len );
+                if( code < 0 || code != (unsigned char)name[i2++] ) return false;
+                i1 += len;
+                } break;
+      default: if( ch1 != (unsigned char)name[i2++] ) return false;
                break;
       }
     }
-  return ( i2 >= name.size() );
+  return i2 >= name.size();
   }
