@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "buffer.h"
 #include "buffer_handle.h"
 #include "block.h"
+#include "encoding.h"
 #include "iso_8859.h"
 #include "menu.h"
 #include "rc.h"
@@ -68,7 +69,7 @@ int clock_limit;		//   0: disable clock
 				// > 1: first line used by menu
 enum { clock_string_size = 5 };
 char clock_string_[clock_string_size+1] = " 0:00";
-char clock_char = ':';		// ' ' or ':' to enable/disable blinking
+bool blink = false;		// enable blinking dots in clock
 
 std::vector< Lines_and_cursor > lines_and_cursor_vector;
 
@@ -160,7 +161,8 @@ const char * alt_key_loop( int key )
   w.show_status_line();
   key = ISO_8859::decontrolize( key );
   last_key.shift( key );
-  if( std::isdigit( key ) ) msg = w.goto_mark( key - '0' );
+  if( ISO_8859::isuchar( key ) && std::isdigit( key ) )
+    msg = w.goto_mark( key - '0' );
   else switch( key )
     {
     case 'A': w.scroll_horizontal( -8 ); break;
@@ -220,7 +222,8 @@ const char * control_k_loop( int key )
   w.show_status_line();
   key = ISO_8859::decontrolize( key );
   last_key.shift( key );
-  if( std::isdigit( key ) ) msg = w.set_mark( key - '0' );
+  if( ISO_8859::isuchar( key ) && std::isdigit( key ) )
+    msg = w.set_mark( key - '0' );
   else switch( key )
     {
     case 'B': Block::set_begin( w.buffer(), w.pointer() );
@@ -319,7 +322,7 @@ const char * control_s_loop( int key, const char * const version_msg )
     case '2':
     case '4':
     case '8': msg = w.show_multichar_value( key - '0', false ); break;
-    case 'B': clock_char = ( clock_char != ':' ) ? ':' : ' '; break;
+    case 'B': blink = !blink; break;
     case 'C': RC::editor_options().show_code_lpos =
                 ( RC::editor_options().show_code_lpos == 1 ) ? 0 : 1;
               show_status_lines(); break;
@@ -351,7 +354,7 @@ const char * user_data( const std::string & s, const int base )
   else
     for( int i = 0; i <= last; ++i )
       {
-      if( base == 10 && !std::isdigit( s[i] ) )
+      if( base == 10 && !std::isdigit( (unsigned char)s[i] ) )
         return "Invalid decimal data";
       if( base == 8 &&  !ISO_8859::isodigit( s[i] ) )
         return "Invalid octal data";
@@ -400,12 +403,18 @@ const char * user_control_char( const int abort_key )
   else
     {
     history.push_back( std::string() );
-    const int size = get_string( prompt2, history, abort_key );
+    const long size = get_string( prompt2, history, abort_key );
     if( size <= 0 ) return 0;
     const std::string & s = history.back();
     if( size >= 2 && s[0] == '\\' )
-      { int len; key = ISO_8859::escape( s, 1, &len );
-        if( key >= 0 && len + 1 != size ) key = -1; }
+      { int len; key = ISO_8859::escape( s, 1, &len, true );	// allow \u
+        if( key >= 0 && len + 1 != size ) key = -1;
+        else if( key >= 128 && ( s[1] == 'u' || s[1] == 'U' ) )
+          { std::string out;
+            Encoding::ucs_to_utf8( key, out );
+            for( unsigned i = 0; i < out.size(); ++i )
+              Window_vector::add_char( (unsigned char)out[i], true );
+            return 0; } }
     else if( s[0] >= '1' && s[0] <= '9' )
       return user_data( s, 10 );
     else if( size >= 3 && s[0] == '0' && ( s[1] == 'x' || s[1] == 'X' ) )
@@ -465,7 +474,7 @@ bool Screen::init()
   {
   initialize_ncurses();
   getmaxyx( stdscr, height_, width_ );
-  if( height_ < 24 || width_ < 80 ) { endwin(); return false; }
+  if( height_ < 25 || width_ < 80 ) { endwin(); return false; }
   std::atexit( dummy_endwin );
   line_buf_size = width_ + 1;
   line_buf_ = new char[line_buf_size];
@@ -493,35 +502,43 @@ char * Screen::line_buf( int * const sizep )
 const char * Screen::clock_string() { return clock_string_; }
 
 
-// Update the clock.
+// Update the clock string and print it if sig.
 extern "C" void Screen::clock_handler( int sig )
   {
+  static time_t min_since_epoch = 0;
   const time_t tt = std::time( 0 );
-  const std::tm * const t = std::localtime( &tt );
-  if( t )
+  const bool time_change = min_since_epoch != tt / 60 || min_since_epoch == 0;
+
+  if( time_change )
     {
-    const int h = t->tm_hour;
-    clock_string_[0] = ( h < 10 ) ? ' ' : '0' + ( h / 10 );
-    clock_string_[1] = '0' + ( h % 10 );
-    clock_string_[2] = ( clock_string_[2] != ':' ) ? ':' : clock_char;
-    clock_string_[3] = '0' + ( t->tm_min / 10 );
-    clock_string_[4] = '0' + ( t->tm_min % 10 );
-    if( sig )
+    min_since_epoch = tt / 60;
+    const std::tm * const t = std::localtime( &tt );
+    if( t )
       {
-      Point old_cursor; save_cursor( old_cursor );
-      if( clock_limit == 1 )
-        out_raw_string( 0, 60, clock_string_, clock_string_size, 0, A_REVERSE );
-      else for( int i = 0; i < Window_vector::windows(); ++i )
-        {
-        const Point cursor = Window_vector::window( i ).clock_position();
-        if( cursor.line >= clock_limit ) break;
-        out_raw_string( cursor.line, cursor.col, clock_string_,
-                        clock_string_size, 0, A_REVERSE );
-        }
-      wmove( stdscr, old_cursor.line, old_cursor.col );
-      wrefresh( stdscr ); alarm( 1 );
+      const int h = t->tm_hour;
+      clock_string_[0] = ( h < 10 ) ? ' ' : ( h / 10 ) + '0';
+      clock_string_[1] = ( h % 10 ) + '0';
+      clock_string_[3] = ( t->tm_min / 10 ) + '0';
+      clock_string_[4] = ( t->tm_min % 10 ) + '0';
       }
     }
+  if( blink ) clock_string_[2] = ( clock_string_[2] != ':' ) ? ':' : ' ';
+  if( sig && ( time_change || blink ) )
+    {
+    Point old_cursor; save_cursor( old_cursor );
+    if( clock_limit == 1 )
+      out_raw_string( 0, 60, clock_string_, clock_string_size, 0, A_REVERSE );
+    else for( int i = 0; i < Window_vector::windows(); ++i )
+      {
+      const Point cursor = Window_vector::window( i ).clock_position();
+      if( cursor.line >= clock_limit ) break;
+      out_raw_string( cursor.line, cursor.col, clock_string_,
+                      clock_string_size, 0, A_REVERSE );
+      }
+    wmove( stdscr, old_cursor.line, old_cursor.col );
+    wrefresh( stdscr );
+    }
+  if( sig ) alarm( 1 );
   }
 
 
@@ -556,9 +573,9 @@ int Screen::wait_kbhit( const int new_clock_limit, const bool get_key,
    stores the result (if any).
    Return the size of the result string, 0 if none, -1 if aborted.
 */
-int Screen::get_string( const std::string & prompt,
-                        std::vector< std::string > & history,
-                        const int abort_key, const bool usetab )
+long Screen::get_string( const std::string & prompt,
+                         std::vector< std::string > & history,
+                         const int abort_key, const bool usetab )
   {
   const int line = height_ - 1, lines = 1;
   save_lines_and_cursor( line, lines );
@@ -567,14 +584,14 @@ int Screen::get_string( const std::string & prompt,
   remove_duplicates( history );
   std::string s( prompt );
   s += history.back();
-  int scol = s.size();
+  long scol = s.size();
   int idx = history.size() - 1;
   int key = 0;
   bool modified = false;
   while( key >= 0 )
     {
-    const int ahead = std::max( 1, std::min( 8, (int)s.size() + 1 - scol ) );
-    const int pos = std::max( 0, scol + ahead - width_ );
+    const int ahead = std::max( 1L, std::min( 8L, (long)s.size() + 1 - scol ) );
+    const long pos = std::max( 0L, scol + ahead - width_ );
     out_raw_string( line, 0, s, width_, pos );
     wmove( stdscr, line, scol - pos );
     do key = wait_kbhit( line ); while( key < 0 );
@@ -585,7 +602,7 @@ int Screen::get_string( const std::string & prompt,
       case   3 : key = -1; break;				// ^C
       case '\t': if( usetab )
                    {
-                   const int namelen = scol - prompt.size();
+                   const long namelen = scol - prompt.size();
                    std::string name( s, prompt.size(), namelen );
                    if( !Menu::file_menu( prompt, name ) ) { key = -1; break; }
                    scol += name.size() - namelen; modified = true;
@@ -604,25 +621,24 @@ int Screen::get_string( const std::string & prompt,
                  }
                  break;
       case 8:							// ^H
-      case KEY_F(1) : Menu::help_menu( 8, KEY_F(1) ); break;
-      case KEY_F(9) : if( Block::valid() )
-                        {
-                        std::string tmp;
-                        Block::bufferp()->to_string( Block::begin(), Block::end(), tmp );
-                        ISO_8859::escapize( tmp );
-                        if( tmp.size() )
-                          { s.insert( scol, tmp ); scol += tmp.size(); modified = true; }
-                        }
-                      break;
+      case KEY_F(1): Menu::help_menu( 8, KEY_F(1) ); break;
+      case KEY_F(9):
+        if( Block::valid() )
+          {
+          std::string tmp;
+          Block::bufferp()->to_string_esc( Block::begin(), Block::end(), tmp );
+          if( tmp.size() )
+            { s.insert( scol, tmp ); scol += tmp.size(); modified = true; }
+          } break;
       case 127          :					// delete
-      case KEY_BACKSPACE: if( scol > (int)prompt.size() )
+      case KEY_BACKSPACE: if( scol > (long)prompt.size() )
                             { s.erase( --scol, 1 ); modified = true; } break;
-      case KEY_DC   : if( scol < (int)s.size() )
+      case KEY_DC   : if( scol < (long)s.size() )
                         { s.erase( scol, 1 ); modified = true; } break;
       case KEY_HOME : scol = prompt.size(); break;
       case KEY_END  : scol = s.size(); break;
-      case KEY_LEFT : if( scol > (int)prompt.size() ) { --scol; } break;
-      case KEY_RIGHT: if( scol < (int)s.size() ) { ++scol; } break;
+      case KEY_LEFT : if( scol > (long)prompt.size() ) { --scol; } break;
+      case KEY_RIGHT: if( scol < (long)s.size() ) { ++scol; } break;
       case 25       : if( s.size() > prompt.size() )		// ^Y
                         { s.resize( prompt.size() ); scol = s.size();
                           modified = true; }		// fall through
