@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -127,11 +127,10 @@ const char * Window::goto_line()
   history.push_back( std::string() );
   if( Screen::get_string( "Go to line (^C to abort): ", history ) <= 0 )
     return 0;
-  const std::string & s = history.back();
   Point c = bhp->cursor;
   ++c.line;
-  if( !RC::parse_relative_long( s, c.line ) || c.line <= 0 )
-    { history.pop_back(); return "Invalid line number"; }
+  if( !RC::parse_relative_long( history.back().c_str(), c.line ) ||
+      c.line <= 0 ) { history.pop_back(); return "Invalid line number"; }
   --c.line;
   if( c.line > bhp->buffer().last_line() ) c.line = bhp->buffer().last_line();
   update_points( c, true, true, from_cursor );
@@ -146,10 +145,9 @@ const char * Window::goto_column()
   history.push_back( std::string() );
   if( Screen::get_string( "Go to column (^C to abort): ", history ) <= 0 )
     return 0;
-  const std::string & s = history.back();
   Point c = bhp->cursor;
   ++c.col;
-  if( !RC::parse_relative_long( s, c.col ) || c.col <= 0 )
+  if( !RC::parse_relative_long( history.back().c_str(), c.col ) || c.col <= 0 )
     { history.pop_back(); return "Invalid column number"; }
   --c.col;
   update_points( c, true, false, from_cursor );
@@ -165,14 +163,14 @@ const char * Window::goto_offset()
   history.push_back( std::string() );
   if( Screen::get_string( "Go to offset (^C to abort): ", history ) <= 0 )
     return 0;
-  const std::string & s = history.back();
-  const bool relative = RC::issigned( s );
+  const char * const ptr = history.back().c_str();
+  const bool relative = RC::issigned( ptr );
   long offset = 0;
-  if( !RC::parse_long( s, offset ) || ( !relative && offset < 0 ) )
+  if( !RC::parse_long( ptr, offset ) || ( !relative && offset < 0 ) )
     { history.pop_back(); return "Invalid offset value"; }
   const int cr = buffer.crlf;
   Point p( 0, 0 );
-  if( relative ) p = bhp->pointer;
+  if( relative ) { p = bhp->pointer; if( cr && buffer[p] == '\n' ) ++p.col; }
   p.col += offset;
   if( offset > 0 )
     {
