@@ -1,5 +1,5 @@
 /* GNU moe - My Own Editor
-   Copyright (C) 2005-2022 Antonio Diaz Diaz.
+   Copyright (C) 2005-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -120,7 +120,7 @@ bool insert_year( std::vector< int > & years, const int year )
 void print_year( Basic_buffer & outbuf, Point & p, int year )
   {
   while( year )
-    { outbuf.pputc( p, '0' + ( year % 10 ) ); outbuf.pprev( p ); year /= 10; }
+    { outbuf.pputc( p, ( year % 10 ) + '0' ); outbuf.pprev( p ); year /= 10; }
   p = outbuf.eof();
   }
 
@@ -205,6 +205,50 @@ bool insert_new_year( const Buffer & buffer, const Point & p0,
   return change;			// new_year inserted or list modified
   }
 
+
+/* Replace constant escape sequences with special characters;
+   "\n" --> '\n' or "\u00E1" --> "á" (but not "\?" --> '?' or "\\" --> '\').
+   If !allow_set, leave "\0".."\9" and "\u<non_hex_digit>" unmodified.
+   "\u" and "\U" may not be specified in sets nor after "\+".
+   The backslash and the right square bracket must be real (not "\x5C").
+*/
+bool deescapize_string( const std::string & in, std::string & out,
+                        const bool allow_set )
+  {
+  bool in_set = false;
+  bool plus = false;
+  out.clear();
+  for( unsigned long i = 0; i < in.size(); )
+    {
+    if( in[i] != '\\' )			// ordinary character
+      { out += in[i]; if( in[i++] == ']' ) in_set = false; continue; }
+    if( i + 1 >= in.size() ) return false;
+    const unsigned char ch = in[i+1];	// character after the backslash
+    if( ch == '\\' ) { out += in[i++]; out += in[i++]; continue; }
+    if( !allow_set && ( std::isdigit( ch ) || ( ch == 'u' &&
+        ( i + 2 >= in.size() || ISO_8859::xvalue( in[i+2] ) < 0 ) ) ) )
+      { out += in[i++]; out += in[i++]; continue; }
+    if( !in_set && ch == '[' )
+      {
+      if( !allow_set || i + 3 >= in.size() ) return false;
+      out += in[i++]; out += in[i++];		// "\["
+      if( in[i] == '^' ) out += in[i++];	// complemented set
+      if( in[i] == ']' ) out += in[i++];	// first character in set
+      in_set = true; continue;
+      }
+    int len, code = ISO_8859::escape( in, i + 1, &len, !in_set && !plus );
+    if( code < 0 )
+      { if( in_set || code != -1 ) return false;
+        plus = ch == '+'; out += in[i++]; out += in[i++]; continue; }
+    if( code == '\\' || code == ']' ) return false;	// reject "\x5C", etc
+    if( code >= 128 && ( ch == 'u' || ch == 'U' ) )
+      Encoding::ucs_to_utf8( code, out );
+    else out += code;
+    i += len + 1;
+    }
+  return !in_set;
+  }
+
 } // end namespace Window_vector
 
 
@@ -274,14 +318,9 @@ const char * Window_vector::change_buffer_name()
   std::string name( buffer.name() );
 
   if( Menu::get_filename( "New buffer name (^C to abort): ", name ) < 0 ||
-      name == buffer.name() )
-    return 0;
+      name == buffer.name() ) return 0;
   const char * const msg = already_being_edited( name, &buffer );
-  if( !msg )
-    {
-    buffer.name( name );
-    Screen::show_status_lines( &buffer );
-    }
+  if( !msg ) { buffer.name( name ); Screen::show_status_lines( &buffer ); }
   return msg;
   }
 
@@ -290,12 +329,11 @@ const char * Window_vector::load_file( const int abort_key )
   {
   std::string name;
   if( Menu::get_filename( "Name of file to edit (^C to abort): ",
-                          name, abort_key ) < 0 )
-    return 0;
+                          name, abort_key ) < 0 ) return 0;
   Screen::show_feedback( "loading..." );
   const int old_handles = Bufhandle_vector::handles();
   int handle;
-  try { handle = Bufhandle_vector::find_or_add_handle( &name, curwin_ + 1 ); }
+  try { handle = Bufhandle_vector::find_or_add_handle( name, curwin_ + 1 ); }
   catch( Buffer::Error & e )
     { data.back().repaint(); return e.msg; }		// remove feedback
   curwin_ = handle;
@@ -318,9 +356,8 @@ const char * Window_vector::save_file( const int abort_key )
   const Buffer & buffer = curwin().buffer();
   std::string name( buffer.name() );		// name may be empty (unnamed)
 
-  if( Menu::get_filename( "Name of file to save (^C to abort): ",
-      name, abort_key ) <= 0 )
-    return 0;
+  if( Menu::get_filename( "Name of file to save (^C to abort): ", name,
+                          abort_key ) <= 0 ) return 0;
   const char * const msg = already_being_edited( name, &buffer );
   if( msg ) return msg;
   bool done;
@@ -556,7 +593,7 @@ const char * Window_vector::read_block()
   if( Menu::get_filename( "Name of file to insert (^C to abort): ", name ) <= 0 )
     return 0;
   try {
-    Buffer tmp( &name );
+    Buffer tmp( name.c_str() );
     if( tmp.empty() ) throw Buffer::Error( "File not found" );
     Point p = w.pointer();
     if( w.buffer().pputb( p, tmp, tmp.bof(), tmp.eof() ) )
@@ -748,6 +785,7 @@ const char * Window_vector::reformat_paragraph()
   if( read_only( buffer ) ) return 0;
 
   Point cursor = w.buffer_handle().cursor;
+  Screen::show_feedback( "reformatting..." );
   try {
     if( buffer.reformat_paragraph( cursor, false ) )
       {
@@ -755,8 +793,9 @@ const char * Window_vector::reformat_paragraph()
       Screen::repaint( &buffer );
       w.update_points( cursor, true, false, Window::from_cursor );
       }
+    else w.repaint();				// remove feedback
     }
-  catch( Buffer::Error & e ) { return e.msg; }
+  catch( Buffer::Error & e ) { w.repaint(); return e.msg; } // remove feedback
   return 0;
   }
 
@@ -773,10 +812,18 @@ const char * Window_vector::reformat_block()
   cursor.line = Block::begin().line;			// keep cursor col
   const bool incomplete = Block::end().col > 0;
   bool change = false;
-  Screen::show_feedback( "reformating..." );
+  Screen::show_feedback( "reformatting..." );
   while( cursor.line < Block::end().line + incomplete &&
          cursor.line < buffer.lines() )
-    if( buffer.blank( cursor.line ) ) ++cursor.line;
+    if( buffer.blank( cursor.line ) )	// remove whitespace from blank lines
+      {
+      const long c = buffer.characters( cursor.line );
+      if( c > 1 || ( c == 1 && cursor.line == buffer.last_line() ) )
+        { if( change ) buffer.force_append(); else buffer.reset_appendable();
+          buffer.pdelb( buffer.bol( cursor ), buffer.eol( cursor ) );
+          change = true; }
+      ++cursor.line;					// keep cursor col
+      }
     else
       {
       try { if( buffer.reformat_paragraph( cursor, change ) ) change = true; }
@@ -868,10 +915,7 @@ const char * Window_vector::decode_base64()
       read_only( buffer ) ) return 0;
 
   std::string in, out;
-  Block::bufferp()->to_string( Block::begin(), Block::end(), in );
-  for( long i = in.size() - 1; i >= 0; --i )
-    if( ISO_8859::isspace( in[i] ) )
-      in.erase( i, 1 );
+  Block::bufferp()->to_string_ws( Block::begin(), Block::end(), in );
   long idx;
   if( !Encoding::base64_decode( in, out, &idx ) )
     {
@@ -1073,7 +1117,7 @@ const char * Window_vector::copyright_update()
     {
     int year = 1900 + t->tm_year;
     while( year )
-      { year_string.insert( year_string.begin(), '0' + ( year % 10 ) );
+      { year_string.insert( year_string.begin(), ( year % 10 ) + '0' );
         year /= 10; }
     }
   history.push_back( year_string );
@@ -1083,7 +1127,8 @@ const char * Window_vector::copyright_update()
   int new_year = 0;
   for( unsigned i = 0; i < year_string.size(); ++i )
     {
-    if( !std::isdigit( year_string[i] ) ) { year_string.clear(); break; }
+    if( !std::isdigit( (unsigned char)year_string[i] ) )
+      { year_string.clear(); break; }
     new_year *= 10; new_year += year_string[i] - '0';
     }
   if( year_string.size() < 4 || year_string[0] == '0' ||
@@ -1181,8 +1226,8 @@ const char * Window_vector::search( const int abort_key, const bool again,
   static std::vector< std::string > search_history;
   static std::vector< std::string > replace_history;
   static std::vector< std::string > options_history;
-  static std::string search_string;
-  static std::string replace_string;
+  static std::string search_string;		// deescapized search string
+  static std::string replace_string;		// deescapized replace string
   static bool backward_opt = false;
   static bool block = false;
   static bool global = false;
@@ -1192,16 +1237,16 @@ const char * Window_vector::search( const int abort_key, const bool again,
   if( !again || search_string.empty() )
     {
     search_history.push_back( s_ini );
-    int size = Screen::get_string( "Find (^C to abort): ", search_history,
-                                   abort_key );
+    long size = Screen::get_string( "Find (^C to abort): ", search_history,
+                                    abort_key );
     if( size <= 0 )
       {
       if( s_ini.size() && search_history.size() ) search_history.pop_back();
       if( search_history.empty() ) search_string.clear();
       return 0;
       }
-    search_string = search_history.back();
-    ISO_8859::deescapize_search_string( search_string );
+    if( !deescapize_string( search_history.back(), search_string, true ) )
+      { search_string.clear(); return "Syntax error in search string"; }
     std::string prompt;
     if( RC::editor_options().ignore_case )
       prompt = "(N)o_ignore_case (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
@@ -1230,10 +1275,12 @@ const char * Window_vector::search( const int abort_key, const bool again,
       {
       replace_string.clear();		// by default replace with nothing
       replace_history.push_back( std::string() );
-      const int len = Screen::get_string( "Replace with (^C to abort): ",
-                                          replace_history, abort_key );
-      if( len < 0 ) { replace = false; return 0; }
-      if( len > 0 ) replace_string = replace_history.back();
+      size = Screen::get_string( "Replace with (^C to abort): ",
+                                 replace_history, abort_key );
+      if( size < 0 ) { replace = false; return 0; }
+      if( size > 0 &&
+          !deescapize_string( replace_history.back(), replace_string, false ) )
+        { search_string.clear(); return "Syntax error in replace string"; }
       }
     }
   if( toggle == 2 ) backward_opt = !backward_opt;	// toggle permanently
